#ifndef CHESSY_CHESSGAME_H
#define CHESSY_CHESSGAME_H

#include <vector>
#include <string>

#include "Board.h"

namespace chessy
{
    class ChessGame;
}


class chessy::ChessGame
{
    Board board;

    bool canCastleQueenSideWhite =  true;
    bool canCastleQueenSideBlack =  true;
    bool canCastleKingSideWhite =   true;
    bool canCastleKingSideBlack =   true;

    short reversibleHalfMoves =     0;

    //! -1 for no en passant possible
    Index enPassant =              -1;

    enum class Turn { WHITE, BLACK };
    //! 0 for white, 1 for black
    Turn turn =                     Turn::WHITE;

    int moveCount =                 0;

    const static char WHITE =       0;
    const static char BLACK =       1;


public:

    ChessGame(const std::string& fenString);

    ChessGame(void) = default;
    ChessGame(const ChessGame&) = default;
    ~ChessGame(void) = default;

    bool isValidMove(const Move& move) const;
    std::vector<Move> getValidMoves(void) const;

    inline const Board& getBoard(void) const { return board; }

    void loadFromFen(const std::string& fenString);
    std::string generateFen(void) const;
};

#endif // CHESSY_CHESSGAME_H

